package app.crescentcash.src.async

import android.os.AsyncTask
import android.view.View
import app.crescentcash.src.MainActivity
import app.crescentcash.src.manager.NetManager
import app.crescentcash.src.manager.UIManager
import app.crescentcash.src.manager.WalletManager
import app.crescentcash.src.utils.PrefsUtil
import org.bitcoinj.core.Address
import org.bitcoinj.core.AddressFactory
import org.bitcoinj.wallet.DeterministicSeed
import org.bitcoinj.wallet.Wallet

class AsyncTaskVerifyWallet(val activity: MainActivity, val cashAcctName: String, val seed: DeterministicSeed) : AsyncTask<Void, Void, Boolean>() {
    lateinit var cashAcctEmoji: String

    override fun doInBackground(vararg params: Void?): Boolean? {
        println("VERIFYING CASH ACCOUNT")
        try {
            val cashAcctAddress = org.bitcoinj.net.NetHelper().getCashAccountAddress(WalletManager.parameters, cashAcctName)
            cashAcctEmoji = NetManager.getCashAccountEmoji(cashAcctName)
            println(cashAcctAddress)
            val accountAddress: Address?

            accountAddress = when {
                Address.isValidCashAddr(WalletManager.parameters, cashAcctAddress) -> AddressFactory.create().getAddress(WalletManager.parameters, cashAcctAddress)
                Address.isValidLegacyAddress(WalletManager.parameters, cashAcctAddress) -> Address.fromBase58(WalletManager.parameters, cashAcctAddress)
                else -> {
                    activity.runOnUiThread { UIManager.showToastMessage(activity, "No address found!") }
                    null
                }
            }

            val tempWallet = Wallet.fromSeed(WalletManager.parameters, seed)

            return if (accountAddress != null) {
                tempWallet.isPubKeyHashMine(accountAddress.hash160)
            } else {
                false
            }
        } catch (e: NullPointerException) {
            println("ERROR")
            e.printStackTrace()
            activity.runOnUiThread { UIManager.showToastMessage(activity, "Cash Account not found.") }
            return false
        }
    }

    override fun onPostExecute(result: Boolean?) {
        super.onPostExecute(result)

        val isAddressMine = result!!
        if (isAddressMine) {
            WalletManager.setupWalletKit(activity, seed, cashAcctName, true)
            val pref = PrefsUtil.prefs.edit()
            pref.putString("cashAccount", cashAcctName)
            pref.putString("cashEmoji", cashAcctEmoji)
            pref.putBoolean("isNewUser", false)
            pref.apply()

            activity.runOnUiThread {
                activity.restore_wallet.visibility = View.GONE
                activity.displayDownloadContent(true)
                activity.displayDownloadContentSlp(true)
                activity.runOnUiThread { UIManager.showToastMessage(activity, "Verified!") }
            }
        } else {
            activity.runOnUiThread { UIManager.showToastMessage(activity, "Verification failed!") }
        }
    }
}